package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.advertising.tencent.base.AdvertisingTencentServer;
import com.zanxiang.advertising.tencent.base.rpc.IUserActionSetRpc;
import com.zanxiang.erp.base.ErpServer;
import com.zanxiang.erp.base.rpc.ISysUserRpc;
import com.zanxiang.game.back.serve.dao.mapper.GameTencentMiniGameUserMapper;
import com.zanxiang.game.back.serve.pojo.dto.GameTencentMiniGameUserDTO;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentMiniGameBackLog;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentMiniGameUser;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.pojo.vo.GameTencentMiniGameUserVO;
import com.zanxiang.game.back.serve.service.IGameTencentMiniGameBackLogService;
import com.zanxiang.game.back.serve.service.IGameTencentMiniGameUserService;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Slf4j
@Service
public class GameTencentMiniGameUserServiceImpl extends ServiceImpl<GameTencentMiniGameUserMapper, GameTencentMiniGameUser>
        implements IGameTencentMiniGameUserService {

    @DubboReference(providedBy = AdvertisingTencentServer.SERVER_DUBBO_NAME)
    private IUserActionSetRpc userActionSetRpc;
    @DubboReference(providedBy = ErpServer.SERVER_DUBBO_NAME)
    private ISysUserRpc sysUserRpc;
    @Autowired
    private IGameTencentMiniGameBackLogService gameTencentMiniGameBackLogService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean userBack(GameTencentMiniGameUser userLog, boolean mustBack) {
        BackStatusEnum backStatus = doCallback(userLog, mustBack);
        return update(new LambdaUpdateWrapper<GameTencentMiniGameUser>()
                .set(GameTencentMiniGameUser::getBackStatus, backStatus.getBackStatus())
                .eq(GameTencentMiniGameUser::getId, userLog.getId())
        );
    }


    @Override
    public IPage<GameTencentMiniGameUserVO> listOfPage(GameTencentMiniGameUserDTO dto) {
        return page(dto.toPage(), new LambdaQueryWrapper<GameTencentMiniGameUser>()
                .eq(StringUtils.isNotBlank(dto.getAgentKey()), GameTencentMiniGameUser::getAgentKey, dto.getAgentKey())
                .eq(null != dto.getGameId(), GameTencentMiniGameUser::getGameId, dto.getGameId())
                .eq(null != dto.getAdAccountId(), GameTencentMiniGameUser::getAdAccountId, dto.getAdAccountId())
                .eq(StringUtils.isNotBlank(dto.getWechatAppId()), GameTencentMiniGameUser::getWechatAppId, dto.getWechatAppId())
                .eq(StringUtils.isNotBlank(dto.getWechatOpenid()), GameTencentMiniGameUser::getWechatOpenid, dto.getWechatOpenid())
                .eq(null != dto.getBackStatus(), GameTencentMiniGameUser::getBackStatus, dto.getBackStatus())
                .orderByDesc(GameTencentMiniGameUser::getCreateTime)
        ).convert(this::toVOSimple);
    }

    @Override
    public boolean doReport(List<Long> userLogIds) {
        listByIds(userLogIds).forEach(userLog -> {
            BackStatusEnum backStatus = doCallback(userLog, true);
            update(new LambdaUpdateWrapper<GameTencentMiniGameUser>()
                    .set(GameTencentMiniGameUser::getBackStatus, backStatus.getBackStatus())
                    .eq(GameTencentMiniGameUser::getId, userLog.getId())
            );
        });
        return true;
    }


    private BackStatusEnum doCallback(GameTencentMiniGameUser userLog, boolean mustBack) {
        boolean isBack = mustBack ? false : gameTencentMiniGameBackLogService.count(new LambdaQueryWrapper<GameTencentMiniGameBackLog>()
                .eq(GameTencentMiniGameBackLog::getGameId, userLog.getGameId())
                .eq(GameTencentMiniGameBackLog::getWechatAppId, userLog.getWechatAppId())
                .eq(GameTencentMiniGameBackLog::getWechatOpenid, userLog.getWechatOpenid())
                .eq(GameTencentMiniGameBackLog::getAdAccountId, userLog.getAdAccountId())
                .eq(GameTencentMiniGameBackLog::getBackStatus, BackStatusEnum.SUCCESS.getBackStatus())
        ) > 0;
        if (isBack) {
            return BackStatusEnum.NO;
        }
        return gameTencentMiniGameBackLogService.userBack(userLog);
    }

    private GameTencentMiniGameUserVO toVOSimple(GameTencentMiniGameUser userLog) {
        if (userLog == null) {
            return null;
        }
        return BeanUtil.copy(userLog, GameTencentMiniGameUserVO.class);
    }
}
